# 函数

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_functions.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_functions.html)

注意

本节是关于从 python 调用 r 函数的。要使 python 函数可由 r 调用，请参见低级函数`rpy2.rinterface.rternalize()`。

`rpy2`的高级接口所公开的 r 函数可用于：

*   像任何常规的 python 函数一样，它们是可调用对象（参见[callable](#robjects-functions-callable)一节）
*   通过他们的方法`rcall()`（见[rcall（）](#robjects-functions-rcall)节）

## 可调用

```
from rpy2.robjects.packages import importr
base = importr('base')
stats = importr('stats')
graphics = importr('graphics')

plot = graphics.plot
rnorm = stats.rnorm
plot(rnorm(100), ylab="random")

```

在遇到名为`na.rm`的 r 参数之前，这一切看起来都很简单。默认情况下，这是通过将 r 参数名中的“.”（点）转换为 python 参数名中的“u”来解决的。

让我们以 R 中的一个例子为例：

```
rank(0, na.last = TRUE)
# or without the implicit namespace:
base::(0, na.last = TRUE)

```

在 python 中，可以写：

```
from rpy2.robjects.packages import importr
base = importr('base')

base.rank(0, na_last = True)

```

Note

在本例中，对象`base.rank`是`functions.SignatureTranslatedFunction`的实例，是`functions.Function`的子类，并且在创建实例期间对参数名进行转换。在创建过程中进行转换显然节省了对参数名执行转换操作的需要，例如在每个函数调用时将`‘.’`替换为`‘_’`，并允许`rpy2`对可能的不明确转换（`R`函数，甚至在基库中，有时在同一函数的签名中同时具有参数名`foo.bar`和`foo_bar`。执行映射的成本在重复调用函数时分摊，因为只有在创建实例时才执行此操作。

如果不需要翻译，可以使用类`functions.Function`。使用该类，使用特殊的 python 语法`**kwargs`是为包含点`‘.’`的 r 函数指定命名参数的一种方法。

我们会注意到，翻译是通过检查 r 函数的签名来完成的，并且只要存在 r 省略号`‘...’`就无法猜测太多。属于`‘...’`的参数需要将其 r 名称传递给`functions.SignatureTranslatedFunction`的构造函数，如下面的示例所示：

```
>>> graphics = importr('graphics')
>>> graphics.par(cex_axis = 0.5)
Warning message:
In function (..., no.readonly = FALSE)  :
"cex_axis" is not a graphical parameter
<Vector - Python:0xa1688cc / R:0xab763b0>
>>> graphics.par(**{'cex.axis': 0.5})
<Vector - Python:0xae8fbec / R:0xaafb850>

```

存在手动指定参数映射的方法：

```
from rpy2.robjects.functions import SignatureTranslatedFunction
STM = SignatureTranslatedFunction
from rpy2.robjects.packages import importr
graphics = importr('graphics')
graphics.par = STM(graphics.par,
                   init_prm_translate = {'cex_axis': 'cex.axis'})

```

```
>>> graphics.par(cex_axis = 0.5)
<Vector - Python:0xa2cc90c / R:0xa5f7fd8>

```

盲目地将参数名中的每一个`‘.’`翻译成`‘_’`目前看来是一种危险的做法，由一个人来决定他/她自己的代码。代码示例演示了如何执行，而不是建议执行以下操作：

```
def iamfeelinglucky(func):
    def f(*args, **kwargs):
        d = {}
        for k, v in kwargs.items():
            d[k.replace('_', '.')] = v
        return func(**d)
    return f

lucky_par = iamfeelinglucky(graphics.par)
lucky_path(cex_axis = 0.5)

```

事情也并非总是那么简单，因为字典的使用并不能确保传递参数的顺序是守恒的。

R 能够自省，并且可以通过函数`formals()`返回函数接受的参数，该函数被建模为`functions.Function`的方法。

```
>>> from rpy2.robjects.packages import importr
>>> stats = importr('stats')
>>> rnorm = stats.rnorm
>>> rnorm.formals()
<Vector - Python:0x8790bcc / R:0x93db250>
>>> tuple(rnorm.formals().names)
('n', 'mean', 'sd')

```

警告

这里还有一个来自 r 的扭曲，一些函数是“特殊的”。rpy2 作为`rpy2.rinterface.SexpClosure`r 对象公开，对象可以是 closxp、builtinsxp 或 specialsxp。但是，只有 closxp 对象将返回非空`formals`。

## `rcall()`

方法`Function.rcall()`是调用底层 r 函数的另一种方法。在使用 R 环境时，必须指定要在其中评估函数的环境。

我们在`plot()`中再次使用该示例：

```
from rpy2.robjects.packages import importr
base = importr('base')
stats = importr('stats')
graphics = importr('graphics')

plot = graphics.plot
rnorm = stats.rnorm

# import R's "GlobalEnv" to evaluate the function
from rpy2.robjects import globalenv

# build a tuple of 2-tuple as arguments
args = (('x', rnorm(100)),)

# run the function in globalenv
plot.rcall(args, globalenv)

```

在上面的例子中，y 轴的标签是从调用（在 r 中，使用函数`deparse()`）中推断出来的，这产生了相当长的标签。这种情况是因为矢量`x`是一个匿名对象，就`R`而言：虽然它有一个 python（`“x”`）的符号，但它没有任何用于`R`的符号。

方法`rcall()`可以通过让一个人使用一个可以将 r 对象绑定到符号（名称）的环境来帮助克服这个问题。虽然可以使用`globalenv`，但是专用的环境可以导致更好的代码划分。

然后上面的调用可以变成：

```
from rpy2.robjects import Environment

# Create an R environment
env = Environment()

# Bind in R the R vector to the symbol "x" and
# in that environment
env['x'] = rnorm(100)

# Build a tuple of pairs (<argument name>, <argument>).
# Note that the argument is a symbol. R will resolve what
# object is associated to that symbol when the function
# is executed.
args = (('x', base.as_symbol('x')),)

# plot
plot.rcall(args, env)

```

## 文档字符串

[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")中定义的 r 函数继承自类`rpy2.rinterface.SexpClosure`，有关函数行为的进一步文档可以在[函数](rinterface.html#rinterface-functions)部分找到。