# R 包

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_rpackages.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_rpackages.html)

## 导入 R 包

在 R 中，对象可以捆绑到包中进行分发。与 Python 模块类似，可以安装包，然后在需要时加载。这是通过 r 函数 _library（）_ 和 _require（）_（将包的命名空间附加到 r`search path`）实现的。

```
from rpy2.robjects.packages import importr
utils = importr("utils")

```

对象`utils`现在是一个名称空间对象，因为它的`__dict__`包含对应于 r 符号的键。例如，R 函数 _data（）_ 的访问方式如下：

```
>>> utils.data
<SignatureTranslatedFunction - Python:0x913754c / R:0x943bdf8>

```

不幸的是，访问 R 符号可能不那么简单，因为 R 符号可能包含在 Python 符号中无效的字符。任何有 R 经验的人都可以添加一个 DOT 的偏好（_。_）。

为了解决这个问题，在导入包的过程中，尝试翻译 R 符号，圆点变为下划线。这与在`rpy`中可以找到的不一样，但有明显的区别：

*   导入包并缓存结果时，将执行一次转换。缓存允许我们执行下面的检查。

*   检查翻译是否掩盖了包中的其他 R 符号（例如，同时存在“print-me”和“print.me”）。如果发生这种情况，就会引发一个`rpy2.robjects.packages.LibraryError`。要避免这种情况，请使用函数`importr()`中的可选参数 _robject_translations_。

    ```
    d = {'print.me': 'print_dot_me', 'print_me': 'print_uscore_me'}
    thatpackage = importr('thatpackage', robject_translations = d)

    ```

*   由于名称空间封装，翻译仅限于一个包，与相对盲目的 RPY 翻译和检索第一个匹配项相比，限制了屏蔽风险。

注意

有人（有时是直言不讳的）担心似乎不必要的麻烦，即不要盲目地将“”翻译为“”，因为 RPY 正在做的是将包中的所有 R 符号翻译为“”。

幸运的是，R 开发团队在 R 的标准库（推荐包/）中提供了一个真实的示例，以最后一次演示这一点：R 包`tools`包含一个函数`package.dependencies`和一个函数`package_dependencies`，具有不同的行为、签名和文档页面。S.

如果使用`rpy2.robjects.packages`，我们将由您决定如何解决这个问题。一种方法是：

```
d = {'package.dependencies': 'package_dot_dependencies',
     'package_dependencies': 'package_uscore_dependencies'}
tools = importr('tools', robject_translations = d)

```

显然，将“.”转换为“_uuu”是不够的，因为 r 符号在 python 符号中可以使用更多非法字符。这些更奇特的符号可以通过`__dict__`访问。

例子：

```
>>> utils.__dict__['?']
<Function - Python:0x913796c / R:0x9366fac>

```

除了翻译 r objects 符号外，r 函数的对象还将其命名参数翻译为类似的方式（在 python 中“.”变为“uu”）。

```
>>> base = importr('base')
>>> base.scan._prm_translate
{'blank_lines_skip': 'blank.lines.skip',
 'comment_char': 'comment.char',
 'multi_line': 'multi.line',
 'na_strings': 'na.strings',
 'strip_white': 'strip.white'}

```

## 将任意 R 代码作为包导入

R 包不是分发代码的唯一方法。根据作者的经验，存在以.r 文件形式循环的 r 代码。

这很可能不是一件好事，但作为一个 Python 开发人员，这也是在围绕代码实现应用程序（比如 Web 服务）的任务中可能得到的。在大多数工作场所，在代码打包之前，您将无法拒绝它；幸运的是，RPY2 正试图使这种情况尽可能简单。

可以将 r 代码置于字符串中，例如.r 文件的内容，并将其包装为 rpy2r 包。如果您得到了不同的 R 文件，则可以将所有这些文件包装到它们自己的类似于包的结构中，从而不必在各自的文件中考虑这些冲突的名称。

```
square <- function(x) {
    return(x^2)
}

cube <- function(x) {
    return(x^3)
}

```

```
from rpy2.robjects.packages import SignatureTranslatedAnonymousPackage

string = """
square <- function(x) {
    return(x^2)
}

cube <- function(x) {
    return(x^3)
}
"""

powerpack = SignatureTranslatedAnonymousPackage(string, "powerpack")

```

r 函数`square`和`cube`可以用`powerpack.square()`和`powerpack.cube`调用。

无包的 R 代码可以从 URL 访问，一些 R 用户只需从 URL 获取。最近的一个用例是从代码存储库（例如 github）中获取文件。

在 stackoverflow 上使用[代码段：](http://stackoverflow.com/questions/7715723/sourcing-r-script-over-https)

```
library(devtools)
source_url('https://raw.github.com/hadley/stringr/master/R/c.r')

```

Note

如果关心计算机安全，您会想一想代码的起源，以及您相信源代码是什么样的。

python 有从 URL 读取数据的实用程序。

```
import urllib2
from rpy2.robjects.packages import SignatureTranslatedAnonymousPackage

bioc_url = urllib2.urlopen('https://raw.github.com/hadley/stringr/master/R/c.r')
string = ''.join(bioc_url.readlines())

stringr_c = SignatureTranslatedAnonymousPackage(string, "stringr_c")

```

对象`stringr_c`将 r 文件中定义的函数封装为类似 rpy2`importr`返回的函数。

```
>>> type(stringr_c)
rpy2.robjects.packages.SignatureTranslatedAnonymousPackage
>>> stringr_c._rpy2r.keys()
['str_join', 'str_c']

```

与第一次显示的 R 代码不同，这并不是在 R 全局环境中写入任何内容。

```
>>> from rpy2.robjects import globalenv
>>> globalenv.keys()
()

```

### r 命名空间

在 R 中，一个`namespace`描述了一些特定的东西，其中符号可以被导出，或者保持在内部。许多最近的 R 包都声明了一个名称空间，但这不是强制性的，尽管在一些 R 开发圈中建议这样做。

名称空间和控制符号导出的能力几年前在 R 中引入，可能是为了解决 R 程序员对符号封装相对缺乏控制的问题。如果不导入，r 中的包就像在 python 中使用的所有包和模块上系统地编写`import *`一样，随着使用的包数量的增加，这将不可避免地产生潜在的问题。

由于默认情况下，python 通常不具有相同的要求，`importr()`公开名称空间中的所有对象，无论它们是否被导出。

### 类图

## 找到 R 符号的来源

当解析一个给定的符号时，知道有效地考虑哪个对象在 R 中是非常重要的，因为附加的包的数量在增加，并且使用名称空间访问器“：”和“：：：”的频率也不高。

函数`wherefrom()`提供了一种查找它的方法：

```
>>> import rpy2.robjects.packages as rpacks
>>> env = rpacks.wherefrom('lm')
>>> env.do_slot('name')[0]
'package:stats'

```

Note

这并不能完全概括，关于环境和包作为环境的更多细节应该在[sexpenvironment](rinterface.html#rinterface-sexpenvironment)部分进行检查。

## 安装/拆卸 R 包

R 附带了一组 _ 推荐的包 _（相当于一个标准库），但其他包的数量也越来越多。

安装这些包可以在 R 中完成，也可以在命令行上使用 R。进行此操作时，应参考 R 文档。

它还可以从 python/rpy2 安装 R 包，并且是一种非交互方式。

```
import rpy2.robjects.packages as rpackages
utils = rpackages.importr('utils')

utils.chooseCRANmirror(ind=1) # select the first mirror in the list

```

如果您使用生物导体：

```
utils.chooseBioCmirror(ind=1) # select the first mirror in the list

```

`choose&lt;organization&gt;mirror`函数设置一个 r 全局选项，该选项指示默认情况下应使用哪个存储库。下一步是简单地调用 r 函数从存储库安装。

```
packnames = ('ggplot2', 'hexbin')
from rpy2.robjects.vectors import StrVector
utils.install_packages(StrVector(packnames))

```

Note

设置默认存储库的全局选项将一直保留，直到 R 进程结束（或更改默认值）。

如果不首先选择镜像而调用`install_packages()`，则需要用户交互选择镜像。

控制大部分事情是可能的；更多信息，应该参考 R 文档。