# 使用 R OOP

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_oop.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_oop.html)

面向对象编程可以用 R 实现，但可以用多种方法实现。除了 _ 官方 _s3 和 s4 系统之外，还有一个丰富的生态系统，其中包含各种对象的替代实现，如 aroma 或 proto。

## S3 对象

S3 对象是默认的 R 对象（即，不是 S4 实例），已经为其添加了属性“class”。

```
>>> x = robjects.IntVector((1, 3))
>>> tuple(x.rclass)
('integer',)

```

使对象 x 成为类 _ 对 _ 的实例，本身继承自 _ 整数 _ 只是设置属性的问题：

```
>>> x.rclass = robjects.StrVector(("pair", "integer"))
>>> tuple(x.rclass)
('pair', 'integer')

```

_s3_ 类的方法只是具有名称（如 name）的 r 函数。lt；class_name&gt；，在运行时从函数调用中的第一个参数进行调度。

例如，函数 _plot.lm_ 绘制类 _lm_ 的对象。调用 _plot（something）_ 会使 r 提取对象的类名 something，并查看函数 _plot 是否在搜索路径中。&lt；something&gt；_ 的 class_ 是否在搜索路径中。

注意

此规则不严格，因为可能存在名称中带有 _ 点 _ 的函数，并且点后面的部分与 S3 类名不对应。

## S4 对象

S4 对象的类定义稍微正式一点，所有实例都属于低级的 R 类型 sexps4。

类方法的定义可以在类被定义之后的任何时候发生（在 Python 世界中，这是一种被称为 _ 猴子修补 _ 或 _ 鸭子穿孔 _ 的实践）。

显然有很多方法可以尝试在 r 类和 python 类之间进行映射，这里提出的方法是让继承`rpy2.rinterface.methods.RS4`的 python 类。

在研究将 r 类作为 python 类反映的自动化方法之前，我们先研究如何在 python 中定义类来反映 r s4 类。我们将 r 类`lmList`放在包`lme4`中，并演示如何为它编写一个 Python 包装器。

Note

与此部分相关的信息也可以作为 Jupyter 笔记本[s4class.ipynb](_static/notebooks/s4class.ipynb)（html render:[s4class.html](_static/notebooks/s4class.html)）

*   [基本处理](generated_rst/s4class.html)
*   [将 s4 类映射到 python 类](generated_rst/s4class.html#mapping-s4-classes-to-python-classes)
    *   [自定义转换](generated_rst/s4class.html#custom-conversion)
    *   [类属性](generated_rst/s4class.html#class-attributes)
    *   [方法](generated_rst/s4class.html#methods)

### 手动 r-in-python 类定义

Note

r 包`lme4`不是与 r 一起分发的，必须安装它才能使本例工作。

首先，需要一些样板代码。我们导入了更高级的接口和函数`rpy2.robjects.packages.importr()`。我们要表示的 R 类在`rpy2`模块和实用程序中定义。

```
import rpy2.robjects as robjects
import rpy2.rinterface as rinterface
from rpy2.robjects.packages import importr

lme4 = importr("lme4")
getmethod = robjects.baseenv.get("getMethod")

StrVector = robjects.StrVector

```

一旦完成，就可以编写 python 类定义。在代码的第一部分中，我们选择了 R 定义方法的静态映射。这样做的好处是速度有点快（因为 S4 调度机制有成本），缺点是在 R 级别修改方法需要刷新相关映射。代码的第二部分是这些映射的包装器，在这里可以在调用 r 方法之前执行 python-to-r 操作。在类定义的最后一部分，定义了一个 _ 静态方法 _。这是实现多态构造函数的一种方法。

```
class LmList(robjects.methods.RS4):
    """ Reflection of the S4 class 'lmList'. """

    _coef = getmethod("coef", 
                      signature = StrVector(["lmList", ]),
                      where = "package:lme4")
    _confint = getmethod("confint", 
                         signature = StrVector(["lmList", ]),
                         where = "package:lme4")
    _formula = getmethod("formula", 
                         signature = StrVector(["lmList", ]),
                         where = "package:lme4")
    _lmfit_from_formula = getmethod("lmList",
                                    signature = StrVector(["formula", "data.frame"]),
                                    where = "package:lme4")

    def _call_get(self):
        return self.do_slot("call")
    def _call_set(self, value):
        return self.do_slot("call", value)
    call = property(_call_get, _call_set, None, "Get or set the RS4 slot 'call'.")

    def coef(self):
        """ fitted coefficients """
        return self._coef(self)

    def confint(self):
        """ confidence interval """
        return self._confint(self)

    def formula(self):
        """ formula used to fit the model """
        return self._formula(self)

    @staticmethod
    def from_formula(formula, 
                     data = rinterface.MissingArg,
                     family = rinterface.MissingArg,
                     subset = rinterface.MissingArg,
                     weights = rinterface.MissingArg):
        """ Build an LmList from a formula """
        res = LmList._lmfit_from_formula(formula, data,
                                         family = family,
                                         subset = subset,
                                         weights = weights)
        res = LmList(res)
        return res

```

现在可以通过将模型指定为`Formula`和数据集来创建`LmList`的实例。

```
sleepstudy = lme4.sleepstudy
formula = robjects.Formula('Reaction ~ Days | Subject')
lml1 = LmList.from_formula(formula, 
                           sleepstudy)

```

上述方法的一个缺点是存储了 r“call”，当我们传递`DataFrame`_sleepstudy_（并且认为它是 r 的匿名结构）时，调用是冗长的：它包含数据帧的显式结构（尝试打印 _lml1_）。随着数据集越来越大，这变得难以接受。另一种方法是将数据帧的列存储到`Formula`的环境中，如下所示：

```
sleepstudy = lme4.sleepstudy
formula = robjects.Formula('Reaction ~ Days | Subject')
for varname in ('Reaction', 'Days', 'Subject'):
    formula.environment[varname] = sleepstudy.rx2(varname)

lml1 = LmList.from_formula(formula)

```

### 自动化的 r-in-python 类定义

S4 系统允许方法的多态定义，也就是说，可以有几个方法具有相同的名称，但参数的数量和类型不同。（这就像 Clojure 的多重方法）。将 r 方法自动可靠地映射到 python 方法需要一些工作，我们选择将方法名与签名中的参数类型连接起来。

使用自动映射非常简单。只需声明具有以下属性的 python 类：

<colgroup><col width="36%"> <col width="18%"> <col width="46%"></colgroup> 
| _ RName 名称 __ | 强制性的 | R 类的名称 |
| _ rpackagename（包装名称）__ | 可选择的 | 声明类的 r 包 |
| _ 收件人翻译 __ | optional | [`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)")翻译 |
| _ 方法翻译 __ | optional | [`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)") to translate |

例子：

```
from rpy2.robjects.packages import importr
stats4 = importr('stats4')
from rpy2.robjects.methods import RS4Auto_Type
# use "six" for Python2/Python3 compatibility
import six

class MLE(six.with_metaclass(RS4Auto_Type)):
  __rname__ = 'mle'
  __rpackagename__ = 'stats4'

```

刚刚定义的类`MLE`具有表示所有槽（S4 术语中的`attributes`）所需的所有属性和方法，以及在声明类时为类定义的方法（请记住，类方法可以随后声明，甚至可以在不同的 R 包中声明）。

## 用户定义类的自动映射

定义了镜像 R 类的 python 类后，可以通过向转换系统添加新规则来自动进行映射（请参见[节将 rpy2 对象映射到任意 python 对象](robjects_convert.html#robjects-conversion)）。