# 数据帧和`dpylr`

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/lib_dplyr.html](https://rpy2.readthedocs.io/en/version_2.8.x/lib_dplyr.html)

## 普里尔

注意

本节可用作 Jupyter 笔记本[dpylr.ipynb](_static/notebooks/dplyr.ipynb)（html render:[dpylr.html](_static/notebooks/dplyr.html)）

```
from functools import partial
from rpy2.ipython import html
html.html_rdataframe=partial(html.html_rdataframe, table_class="docutils")

```

## python 中的 dplyr

我们需要两件事：

1-一个数据帧（使用 R 的一个演示数据集）。

```
from rpy2.robjects.packages import importr, data
datasets = importr('datasets')
mtcars_env = data(datasets).fetch('mtcars')
mtcars = mtcars_env['mtcars']

```

除此之外，由于本教程在笔记本中，我们初始化了 R 对象的 HTML 呈现（R 数据帧的漂亮显示）。

```
import rpy2.ipython.html
rpy2.ipython.html.init_printing()

```

2-双键

```
from rpy2.robjects.lib.dplyr import DataFrame

```

有了这个，我们可以选择链接（d3 样式）

```
dataf = (DataFrame(mtcars).
         filter('gear>3').
         mutate(powertoweight='hp*36/wt').
         group_by('gear').
         summarize(mean_ptw='mean(powertoweight)'))

dataf

```

<emph>DataFrame</emph> with 2 rows and 2 columns:

|  |  | 齿轮 | 平均值 |
| --- | --- | --- | --- |
| 0 | 1 个 | 4.0 节 | 1237.126649803169 年 |
| 1 个 | 二 | 5.0 条 | 2574.0331639315027 |

或管道（Magrittr 风格）。

```
from rpy2.robjects.lib.dplyr import (filter,
                                     mutate,
                                     group_by,
                                     summarize)

dataf = (DataFrame(mtcars) >>
         filter('gear>3') >>
         mutate(powertoweight='hp*36/wt') >>
         group_by('gear') >>
         summarize(mean_ptw='mean(powertoweight)'))

dataf

```

<emph>DataFrame</emph> with 2 rows and 2 columns:

|  |  | gear | mean_ptw |
| --- | --- | --- | --- |
| 0 | 1 | 4.0 | 1237.1266499803169 |
| 1 | 2 | 5.0 | 2574.0331639315027 |

传递给 dpylr 函数的字符串被计算为表达式，就像在 r 中使用 dpylr 时一样。这意味着在写入`mean(powertoweight)`时使用 r 函数`mean()`。

不过，使用 python 函数并不太困难。我们可以直接从 R 调用 python。要实现这一点，我们只需使用 decorator`rternalize`。

```
# Define a python function, and make
# it a function R can use through `rternalize`
from rpy2.rinterface import rternalize
@rternalize
def mean_np(x):
    import numpy
    return numpy.mean(x)

# Bind that function to a symbol in R's
# global environment
from rpy2.robjects import globalenv
globalenv['mean_np'] = mean_np

# Write a dplyr chain of operations,
# using our Python function `mean_np`
dataf = (DataFrame(mtcars) >>
         filter('gear>3') >>
         mutate(powertoweight='hp*36/wt') >>
         group_by('gear') >>
         summarize(mean_ptw='mean(powertoweight)',
                   mean_np_ptw='mean_np(powertoweight)'))

dataf

```

<emph>DataFrame</emph> with 2 rows and 3 columns:

|  |  | gear | 平均值 | mean_ptw |
| --- | --- | --- | --- | --- |
| 0 | 1 | 4.0 | 1237.12664980317 | 1237.1266499803169 |
| 1 | 2 | 5.0 | 2574.0331639315023 号 | 2574.0331639315027 |

也可以在不需要将自定义函数置于 R 的全局环境中的情况下执行此操作。

```
del(globalenv['mean_np'])

```

```
from rpy2.robjects.lib.dplyr import StringInEnv
from rpy2.robjects import Environment
my_env = Environment()
my_env['mean_np'] = mean_np

dataf = (DataFrame(mtcars) >>
         filter('gear>3') >>
         mutate(powertoweight='hp*36/wt') >>
         group_by('gear') >>
         summarize(mean_ptw='mean(powertoweight)',
                   mean_np_ptw=StringInEnv('mean_np(powertoweight)',
                                           my_env)))

dataf

```

<emph>DataFrame</emph> with 2 rows and 3 columns:

|  |  | gear | mean_np_ptw | mean_ptw |
| --- | --- | --- | --- | --- |
| 0 | 1 | 4.0 | 1237.126649980317 | 1237.1266499803169 |
| 1 | 2 | 5.0 | 2574.0331639315023 | 2574.0331639315027 |

**注**：rpy2 与 dplyr 的接口正在实现对（non-？）的修复第 1323 期（[https://github.com/hadley/dplyr/issues/1323](https://github.com/hadley/dplyr/issues/1323)）

只要数据在表中，就可以直接使用转换到 SQL 的无缝转换。因为我们正在提升`dplyr`的原始实现，所以它 _ 只起作用 _。

```
from rpy2.robjects.lib.dplyr import dplyr
# in-memory SQLite database broken in dplyr's src_sqlite
# db = dplyr.src_sqlite(":memory:")
import tempfile
with tempfile.NamedTemporaryFile() as db_fh:
    db = dplyr.src_sqlite(db_fh.name)
    # copy the table to that database
    dataf_db = DataFrame(mtcars).copy_to(db, name="mtcars")
    res = (dataf_db >>
           filter('gear>3') >>
           mutate(powertoweight='hp*36/wt') >>
           group_by('gear') >>
           summarize(mean_ptw='mean(powertoweight)'))
    print(res)
#

```

```
Source: sqlite 3.8.6 [/tmp/tmp1zb7gwm3]
From: <derived table> [?? x 2]

    gear mean_ptw
   (dbl)    (dbl)
1      4 1237.127
2      5 2574.033
..   ...      ...

```

因为我们在操纵 R 对象，所以对 R 可用的任何东西都对我们可用。如果我们希望看到生成的 SQL 代码：

```
print(res.rx2("query")["sql"])

```

```
<SQL> SELECT "gear", "mean_ptw"
FROM (SELECT "gear", AVG("powertoweight") AS "mean_ptw"
FROM (SELECT "mpg", "cyl", "disp", "hp", "drat", "wt", "qsec", "vs", "am", "gear", "carb", "hp" * 36.0 / "wt" AS "powertoweight"
FROM "mtcars"
WHERE "gear" > 3.0) AS "zzz1"
GROUP BY "gear") AS "zzz2"

```

如果起点是熊猫数据帧，请执行以下操作并重新开始。

```
from rpy2.robjects import pandas2ri
from rpy2.robjects import default_converter
from rpy2.robjects.conversion import localconverter
with localconverter(default_converter + pandas2ri.converter) as cv:
    mtcars = mtcars_env['mtcars']
    mtcars = pandas2ri.ri2py(mtcars)
print(type(mtcars))

```

```
<class 'pandas.core.frame.DataFrame'>

```

使用本地转换器，我们还可以从熊猫数据帧转到我们的 DPLYR 增强 R 数据帧。

```
with localconverter(default_converter + pandas2ri.converter) as cv:
    dataf = (DataFrame(mtcars).
             filter('gear>=3').
             mutate(powertoweight='hp*36/wt').
             group_by('gear').
             summarize(mean_ptw='mean(powertoweight)'))

dataf

```

<emph>DataFrame</emph> with 3 rows and 2 columns:

|  |  | gear | mean_ptw |
| --- | --- | --- | --- |
| 0 | 1 | 3.0 条 | 1633.989574118287 年 |
| 1 | 2 | 4.0 | 1237.1266499803169 |
| 二 | 三 | 5.0 | 2574.0331639315027 |

**重用。把事情做好。不要重新实现。[高温 1]**

## 提迪尔

Note

本节可用作 Jupyter 笔记本[tidyr.ipynb](_static/notebooks/tidyr.ipynb)（html render:[tidyr.html](_static/notebooks/tidyr.html)）

```
from functools import partial
from rpy2.ipython import html
html.html_rdataframe=partial(html.html_rdataframe, table_class="docutils")

```

## 巨蟒中的蒂蒂尔

```
from rpy2.robjects.lib.tidyr import DataFrame

```

（注：`dplyr`由`tidyr`隐式使用。）

In addition to that, and because this tutorial is in a notebook, we initialize HTML rendering for R objects (pretty display of R data frames).

```
import rpy2.ipython.html
rpy2.ipython.html.init_printing()

```

```
from collections import OrderedDict
from rpy2.robjects.vectors import (StrVector,
                                   IntVector)
dataf = DataFrame(OrderedDict(x=StrVector(("a", "b", "b")),
                              y=IntVector((3, 4, 5)),
                          z=IntVector((6, 7, 8))))
dataf

```

<emph>DataFrame</emph> with 3 rows and 3 columns:

|  |  | Z 轴 | X | 是 |
| --- | --- | --- | --- | --- |
| 0 | 1 | 6 | 一 | 三 |
| 1 | 2 | 7 | B | 4 |
| 2 | 3 | 8 个 | b | 5 个 |

```
dataf.spread('x', 'y')

```

<emph>DataFrame</emph> with 3 rows and 3 columns:

|  |  | z | 一 | B |
| --- | --- | --- | --- | --- |
| 0 | 1 | 6 | 3 | 不适用 |
| 1 | 2 | 7 | NA | 4 |
| 2 | 3 | 8 | NA | 5 |

**Reuse. Get things done. Don’t reimplement.**