# 内存管理和垃圾收集

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/rinterface-memorymanagement.html](https://rpy2.readthedocs.io/en/version_2.8.x/rinterface-memorymanagement.html)

r 对象的跟踪（r 的 c-api 中的`SEXP`与 python 不同，因为它不涉及引用计数。它使用的是名为的 at 属性（更多内容见下文），并且只考虑未通过包含在其他 r 对象中而保留的集合对象（对于浮动对象，r 的 c-api 有两个函数`R_PreserveObject()`和`R_ReleaseObject()`，这两个函数的作用仅限于将对象放置在名为[ht]的容器中。G3 页）。

## 参考计数

RPY2 使用自己的引用计数系统，以便将 r 与 python 桥接起来，并尽可能保持 python 用户熟悉的旁路引用方法。

r 对象在 rpy2 中使用的次数，因此不受垃圾收集的保护，可以从 python 获得（显然是只读的）：

```
>>> import rpy2.rinterface as ri
>>> ri.initr()
>>> x = ri.IntSexpVector([1,2,3])
>>> x.__sexp_refcount__
1

```

该计数器将在每次创建对它的新 python 引用时递增。

```
>>> letters = ri.baseenv['letters']
>>> letters.__sexp_refcount__
1
>>> letters_again = ri.baseenv['letters']
>>> # check that the R ID is the same
>>> letters_again.rid == letters.rid
True
>>> # reference count has increased
>>> letters_again.__sexp_refcount__
2
>>> letters.__sexp_refcount__
2

```

注意

属性`rid`只是包含 r 对象的 r 定义的 C 结构所在的内存地址。

可以通过调用`rpy2.rinterface.protected_rids()`来获取 RPY2 保护不受垃圾收集影响的所有 R ID 的列表及其引用计数。

我们可以检查我们的 python 对象`x`是否确实被列为受垃圾收集保护的对象（但它没有绑定到 r 中的任何符号-就 r 而言，它就像一个匿名变量）：

```
>>> x.rid in (elt[0] for elt in ri.protected_rids())
True

```

也可以使用 python/rpy2 对象的数量来保护 r 对象不被垃圾收集。

```
>>> [elt[1] for elt in ri.protected_rids() if elt[0]==x.rid]
[1]

```

Note

确切的计数将取决于当前 python 进程所发生的情况，即 r 对象是否已被 rpy2 跟踪。

将 rpy2 对象绑定到新的 python 符号不会增加计数（因为 python 知道这两个对象是相同的，而 r 没有参与其中）：

```
>>> y = x
>>> [elt[1] for elt in ri.protected_rids() if elt[0]==x.rid]
[1]

```

另一方面，通过 rpy2 构造函数显式地重新包装 r 对象将使计数增加一：

```
>>> z = ri.IntSexpVector(x)
>>> [elt[1] for elt in ri.protected_rids() if elt[0]==x.rid]
[2]
>>> x.rid == z.rid
True

```

在最后一种情况下，python 不知道这两个对象指向同一个底层 r 对象，这种机制旨在防止 r 对象过早地进行垃圾收集。

```
>>> del(x); del(y) # remember that we did `y = x`
>>> [elt[1] for elt in ri.protected_rids() if elt[0]==z.rid]
[1]

```

为了实现这一点，并接近 Python 中的传递引用方法，给定 r 对象的`SexpObject`不是表示它的 Python 对象的一部分。python 对象只保存对它的引用，每次创建指向给定 r 对象（由它的`SEXP`标识）的 python 对象时，它的 rpy 计数器都会递增。

rpy2 对象（r 对象的代理）被实现为一个常规的 python 对象，它附加了一个`SexpObject`指针。

```
typedef struct {
    PyObject_HEAD
    SexpObject *sObj;
} PySexpObject;

```

胶囊本身的跟踪是保护对象不受 R 端或 Python 端垃圾收集的保护。

```
>>> letters_cstruct = letters.__sexp__
>>> del(letters, letters_again)

```

在胶囊被删除后，底层的 r 对象可用于收集（该特定对象不会被删除，因为 r 本身将跟踪它作为基本包的一部分）。

```
>>> del(letters_cstruct)

```

## R 物体胶囊

使用属性`__sexp__`（python`capsule`），可以将`SexpObject`作为（相对）不透明的 C 结构传递。

在场景的后面，胶囊是一个单独的：给定一个 r 对象，它是由第一个包装它的 python（rpy2）对象创建的，当其他 python 对象暴露它时，计数器也会增加和减少。

在 C 级，`struct``SexpObject`定义为：

*   python 端的引用计数
*   R 端可能的未来参考计数（当前未使用）
*   指向 R`SEXPREC`的指针

```
typedef struct {
    Py_ssize_t pycount;
    int rcount;
    SEXP sexp;
} SexpObject;

```

胶囊用于为 RPY2 中基于继承的 R 对象的一般设计提供一种相对安全的成分，比如风味，但是如果需要访问底层的 R`SEXP`对象，仍然可以访问它。下面的示例演示了一种不编写任何 C 代码的方法：

```
import ctypes

# Python C API: get the capsule name (of a capsule object)
pycapsule_getname=ctypes.pythonapi.PyCapsule_GetName
pycapsule_getname.argtypes = [ctypes.py_object,]
pycapsule_getname.restype=ctypes.c_char_p

# Python C API: return whether a Python objects is a valid capsule object
pycapsule_isvalid=ctypes.pythonapi.PyCapsule_IsValid
pycapsule_isvalid.argtypes=[ctypes.py_object, ctypes.c_char_p]
pycapsule_isvalid.restype=ctypes.c_bool

# Python C API: return the C pointer
pycapsule_getpointer=ctypes.pythonapi.PyCapsule_GetPointer
pycapsule_getpointer.argtypes=[ctypes.py_object, ctypes.c_char_p]
pycapsule_getpointer.restype=ctypes.c_void_p

class SexpObject(ctypes.Structure):
    """ C structure SexpObject as defined in the C
        layer of rpy2\. """
    _fields_ = [('pycount', ctypes.c_ssize_t),
                ('rcount', ctypes.c_int),
                ('sexp', ctypes.c_void_p)]

# Function to extract the pointer to the underlying R object
# (*SEXPREC, that is SEXP)
RPY2_CAPSULENAME=b'rpy2.rinterface._rinterface.SEXPOBJ_C_API'
def get_sexp(obj):
    assert pycapsule_isvalid(obj, RPY2_CAPSULENAME)
    void_p=pycapsule_getpointer(obj, RPY2_CAPSULENAME)
    return ctypes.cast(void_p, ctypes.POINTER(SexpObject).contents.sexp

```

```
from rpy2.rinterface import globalenv
# Pointer to SEXPREC for the R Global Environment
sexp=get_sexp(globalenv)

```

以这种方式更改`SexpObject`中的`SEXP`是不明智的，因为这样做可能会混淆 rpy2 中的对象跟踪，最终导致 segfault。（我没想太久。可能是目标跟踪比它想象的更为强大。请注意。）

## R 的名字

只要严格应用传递值范式，垃圾收集就很简单，因为对象只存在于它们声明的范围内，但是 R 对此进行了细微的修改，以尽量减少内存使用。每个 R 对象都有一个附加的属性`Sexp.named`，表示需要复制该对象。

```
>>> import rpy2.rinterface as ri
>>> ri.initr()
0
>>> ri.baseenv['letters'].named
0

```

现在，我们将 r base 名称空间中的向量 _ 字母 _ 分配给 r globalenv 名称空间中的变量 _mine_。

```
>>> ri.baseenv['assign'](ri.StrSexpVector(("mine", )), ri.baseenv['letters'])
<rpy2.rinterface.SexpVector - Python:0xb77ad280 / R:0xa23c5c0>
>>> tuple(ri.globalenv)
("mine", )
>>> ri.globalenv["mine"].named
2

```

名为的 _ 是 2，用于向**r**指示，如果对对象执行任何类型的修改，则应复制 _ 的 _。该副本将是 R 中修改范围的本地副本。_