# 客户端服务器

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/server.html](https://rpy2.readthedocs.io/en/version_2.8.x/server.html)

r server 目前是为 R 寻找服务器解决方案时的默认解决方案，但是`rpy2`可以用来非常容易地开发自己的服务器，以满足特定的需求。例如，这些要求可以包括访问限制、安全模型、R 引擎子集的访问、将作业分配给其他服务器，这些都是当前使用 R 服务难以实现或不可能实现的。

`pyRserve`包处理了从 python 到 r server 的连接，尽管它释放了一个连接来处理 r 服务器，但也限制了一个连接使用 rserver。

## 简单的基于套接字的服务器和客户端

### 服务器

使用 python 的 socket server 模块，在给定端口上监听要作为 r 代码计算的字符串的简单 socket 服务器的实现非常简单。

我们的示例服务器将位于包含以下代码的文件`rpyserve.py`中。

```
import socketserver
import sys
import rpy2.robjects as robjects

class MyTCPHandler(socketserver.StreamRequestHandler):

    def handle(self):
        # verbose server
        print("%s wrote:" % self.client_address[0])

        # self.rfile is a file-like object created by the handler;
        # we can now use e.g. readline() instead of raw recv() calls
        encoding = self.rfile.readline().strip()
        encoding = str(encoding, 'ASCII')
        print('    encoding: %s' % encoding)

        size = int.from_bytes(self.rfile.read(8), 'little')
        print('    size: %i bytes' % size)

        rcv = self.rfile.read(size)
        rcv = str(rcv, encoding) 

        # verbose server
        print('    R code string:')
        print(rcv)

        # evaluate the data passed as a string of R code
        results = robjects.r(rcv)

        # return the result of the evaluation as a string
        # to the client
        results = bytes(str(results), encoding)
        size_res = len(results).to_bytes(8, 'little')
        print('    Result size: %i' % len(results))
        self.wfile.write(size_res +
                         results)

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--port', 
                        type=int,
                        default=8080,
                        help='port')
    parser.add_argument('--hostname',
                        default='localhost')
    options = parser.parse_args()

    # Create the server, binding to localhost on port 9999
    server = socketserver.TCPServer((options.hostname, options.port),
                                    MyTCPHandler)

    print('Server listening on %s:%i' % (options.hostname, options.port))
    # Activate the server; this will keep running until you
    # interrupt the program with Ctrl-C
    server.serve_forever()

```

然后在端口 9090 上运行侦听服务器：

```
python rpyserve.py --hostname localhost

```

### 顾客

使用 python 的 socket 模块，实现客户机同样简单。我们将代码写入一个文件`rpyclient.py`：

```
import socket
import sys
import locale
import argparse

def send_R_code(rcode, hostname, port):
    """
    Evaluate the R code in `rcode` (on a possibly remote machine)
    """
    # Create a socket (SOCK_STREAM means a TCP socket)
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # Connect to server and send data
    sock.connect((hostname, port))
    size_send = len(rcode).to_bytes(8, 'little')
    sock.send(bytes(encoding, 'ASCII') + b'\n' + \
              size_send + \
              rcode)

    # Receive data from the server and shut down
    print("Received:")
    size = int.from_bytes(sock.recv(8), 'little') # 64 bits max
    print("    size: %i bytes" % size)
    received = sock.recv(size)
    sock.close()
    print("    R output:")
    print(str(received, encoding))

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--port',
                        type=int,
                        default=8080)
    parser.add_argument('--hostname',
                        default='localhost')

    options = parser.parse_args()

    # read R code from STDIN
    rcode = sys.stdin.read()
    encoding = locale.getlocale()[1]
    rcode = bytes(rcode, encoding)

    send_R_code(rcode, options.hostname, options.port)

```

如前一节所述，在本地服务器上评估 R 代码，然后监听端口 9090：

```
echo 'R.version' | python rpyclient.py --hostname localhost

```

在本例中，客户机正在查询 R 版本。