# 性能

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/performances.html](https://rpy2.readthedocs.io/en/version_2.8.x/performances.html)

## 优化性能

### 内存使用

r 对象生活在 r 内存空间中，其大小不为 Python 所知，因此，当涉及大型对象时，似乎 Python 不总是足够频繁地进行垃圾收集。当大型对象在循环中被覆盖时，这有时会导致暂时增加内存使用量，尽管达到系统内存限制似乎会触发垃圾收集，但可能希望显式触发收集。

```
import gc
gc.collect()

```

作为一个具体的例子，考虑下面的代码。这已经在某个地方使用了一个独特的基准 python-to-r 桥，不幸的是，没有考虑到 python 和 r 各自的垃圾收集机制的特殊性。基准测试的结果发生了巨大的变化，可能会使 RPY2 成为最快、最节省内存和最通用的 python-to-r 桥。

```
import rpy2.robjects
import gc

r = rpy2.robjects.r

r("a <- NULL")
for i in range(20):
    rcode = "a <- rbind(a, seq(1000000) * 1.0 * %d)" % i
    r(rcode)
    print r("sum(a)")
    # explicit garbage collection
    gc.collect()

```

### 低电平接口

高级层[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")带来了很多便利，比如类映射和接口，但显然在性能方面存在成本。这种代价对于常见的用法是可以抵消的，但是计算密集型的程序通过 python-to-r 桥的方式来回运行一段非常长的时间就会注意到它。

对于这些情况，[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")低级层使程序员更接近 r 的 C 级接口，使 rpy2 比 r 代码本身更快，如下所示。

## 一个简单的基准

作为一个简单的基准，我们采用了一个函数，它可以将数值向量中的所有元素相加。

在纯 R 中，代码如下：

```
function(x)
{
  total = 0;
  for (elt in x) {
    total <- total + elt
  } 
}

```

在纯 Python 中，这类似于：

```
def python_sum(x):
    total = 0.0
    for elt in x:
        total += elt
    return total

```

R 显然有一个向量化函数`sum()`调用底层的 C 代码，但是基准测试的目的是测量纯 R 代码的运行时间。

我们在不同类型的序列上运行这个函数（长度相同）

```
    n = 20000
    x_list = [random.random() for i in range(n)]
    module = None
    if kind == "array.array":
        import array as module
        res = module.array('f', x_list)
    elif kind == "numpy.array":
        import numpy as module
        res = module.array(x_list, 'f')
    elif kind == "FloatVector":
        import rpy2.robjects as module
        res = module.FloatVector(x_list)
    elif kind == "SexpVector":
        import rpy2.rinterface as module
        module.initr()
        res = module.IntSexpVector(x_list)
    elif kind == "list":
        res = x_list
    elif kind == "R":
        import rpy2.robjects as module
        res = module.rinterface.IntSexpVector(x_list)
        module.globalenv['x'] = res
        res = None

```

运行时间总结如下图所示。

[![_static/benchmark_sum.png](_static/benchmark_sum.png)](_static/benchmark_sum.png)

迭代[`list`](http://docs.python.org/library/functions.html#list "(in Python v2.7)")可能是最快的，这解释了为什么在纯 Python 中这种类型的 sum 实现是最快的。对于遍历向量/列表，python 比 r 快得多。

测量各个斜坡，并使用 R 代码的斜坡作为参考，我们得到相对加速，即代码运行速度快多少倍。

<colgroup><col width="21%"> <col width="64%"> <col width="14%"></colgroup> 
| 功能 | 顺序 | 加速 |
| --- | --- | --- |
| 内置 python | 数组.array | 28.94 岁 |
| builtin python | 浮动矢量 | 0.34 分 |
| builtin python | 列表 | 47.94 美元 |
| builtin python | 数字数组 | 1.00 美元 |
| builtin python | 六分仪 | 11.74 条 |
| 纯蟒蛇 | array.array | 8.29 条 |
| pure python | FloatVector | 0.34 |
| pure python | list | 9 月 93 日 |
| pure python | numpy.array | 0.91 美元 |
| pure python | SexpVector | 5.10 条 |
| R 编译 | R compiled | 5.16 条 |
| R | R | 1.00 |
| 减少 python | array.array | 3.45 条 |
| reduce python | FloatVector | 0.30 分 |
| reduce python | list | 3.54 条 |
| reduce python | numpy.array | 0.93 分 |
| reduce python | SexpVector | 3 点 |

对象一的迭代速度非常重要，而最差的执行者是`rpy2.robjects.vectors.FloatVector`，比纯 r 慢了几乎两倍。这是意料之中的，因为迭代依赖于 r 级机制，必须对其添加使用更高级别接口的惩罚。另一方面，使用一个`rpy2.rinterface.SexpVector`提供了一个令人印象深刻的加速，使 R 到 RPY2 的使用比从 R 本身使用 R 更快。这并非意外，因为较低级别的接口更接近 C API for R。由于将`rpy2.robjects.vectors.FloatVector`转换回其父类`rpy2.rinterface.SexpVector`非常简单，因此我们有一种机制，允许 rpy2 以比 r 更快的速度在 r 对象上运行代码。这也意味着，than`rpy2`比其他的 python-to-r 桥更快，在考虑代码的执行时，它将委托 r 对所有代码进行评估。使用`rpy2`从 python 到 r 和 back 的遍历速度也将比使用基于管道的解决方案或基于 rserver 的解决方案更快。

更让人吃惊的是，遍历`numpy.array`的速度比纯 r 略快，比使用`rpy2.rinterface.SexpVector`的速度慢。这是因为后者的子集机制保持了更轻的重量，在需要时提供速度。另一方面，访问`rpy2.robjects.vectors.FloatVector`速度较慢，因为该接口当前是在纯 Python 中实现的，而它是在 C 中用于`numpy.array`。

最后，从前面的基准来看，应该注意到 python 和 r 有一个内置函数 _sum_，调用 C 编译代码，并比较它们的性能。